# Changelog

## v16.2.7
### .NET API
- fix `IOException` that would occur on some systems when `StartScanning` or `GetDiagnosticProfile` was called

## v16.2.6
### .NET API
- fix incorrect phase timings if `SetWindow` was called with a larger window after `Connect`

## v16.2.5
### .NET API
- fix `ScanHead.IsConnected` and `ScanSystem.IsConnected` not being accurate when a scan head gets disconnected from the network
- throw exceptions for methods that try to communicate with a scan head if it has been disconnected from the network, namely `ScanHead.TryTakeNextProfile` and `ScanSystem.TryTakeFrame`
- fix frame scanning not properly resetting sequence number when stopping and starting scans back-to-back
- add `ScanHead.IsScanning` property

## v16.2.4
### .NET API
- fix delay in frame scanning when not all phaseable elements of a scan head are present in the phase table

### C API
- fix delay in frame scanning when not all phaseable elements of a scan head are present in the phase table

## v16.2.3
### JS-50 Firmware
- improvement to internal calibration data

## v16.2.2
### C API
- fix bug in profile queues resulting in incorrect profile available count

## v16.2.1
### .NET API
- updated API examples
- removed `Newtonsoft.Json` dependency
- obsoleted `ScanHeadConfiguration.SetCameraExposureTime` method
- obsoleted `ScanHead.GetDiagnosticCameraImage` method overloads without explicit camera exposure argument

### C API
- updated API examples
- fix bug reading raw profiles from `jsScanHeadGetRawProfiles`
- add new `state` variable to `jsDiscovered` struct
- deprecated `camera_exposure_time_min_us`, `camera_exposure_time_max_us`, and `camera_exposure_time_def_us` fields in `jsScanHeadConfiguration` struct
- deprecated `jsScanHeadGetDiagnosticImageCamera`, `jsScanHeadGetDiagnosticImageLaser`, and `jsScanHeadGetDiagnosticImage` functions

### JS-50 Firmware
- fix bug in scan head discovery causing status to always report `CONNECTED`

### FirmwareUpdate
- speed up firmware update process
- add scan head state column

## v16.2.0
### .NET API
- add support for JS-50 Z820 and JS-50 Z830
- add support for frame scanning
- add support for presending configuration data to scan heads
- add new frame scanning interface `IFrame`
- add new `ScanningMode` enum argument to `ScanSystem.StartScanning`
- add new method `ScanSystem.TryTakeFrame`
- add new configuration methods `ScanSystem.PreSendConfiguration` and `ScanSystem.IsConfigured`
- deprecate `IProfile.PacketsExpected` and `IProfile.PacketsReceived`

### C API
- add support for JS-50 Z820 and JS-50 Z830
- add support for frame scanning
- add support for presending configuration data to scan heads
- add new frame scanning function macros `jsProfileIsValid` and `jsRawProfileIsValid`
- add new frame scanning functions `jsScanSystemStartFrameScanning`, `jsScanSystemGetProfilesPerFrame`, `jsScanSystemWaitUntilFrameAvailable`, `jsScanSystemIsFrameAvailable`, `jsScanSystemClearFrames`, `jsScanSystemGetFrame`, and `jsScanSystemGetRawFrame`
- add new initialization functions `jsProfileInit` and `jsRawProfileInit`
- add new encoder function `jsScanSystemGetEncoder`
- add new configuration functions `jsScanSystemConfigure` and `jsScanSystemIsConfigured`
- deprecate `packets_received` and `packets_expected` fields in `jsProfile` in `jsRawProfile`

### JS-50 Firmware
- add support for JS-50 Z820 and JS-50 Z830

## v16.1.12
### JS-50 Firmware
- add partial support for JS-50 Z820 and JS-50 Z830

## v16.1.11
### JS-50 Firmware
- add support for non-volatile camera correction

## v16.1.10
### JS-50 Firmware
- improvement to internal memory

## v16.1.9
### C API
- fix bug in TCP networking code causing high CPU loads

## v16.1.8
### JS-50 Firmware
- fixed bug that could prevent internal calibration from loading

## v16.1.7
### .NET API
- fix bug that prevented client with multiple network interfaces from connecting to scan heads
- add new property to `ScanHead` named `ClientIpAddress` to get the `IPAddress` of the connected client network interface

### C API
- fix bug that prevented client with multiple network interfaces from connecting to scan heads
- add new fields `client_name_str`, `client_ip_addr`, and `client_netmask` to `jsDiscovered` struct

### JS-50 Firmware
- fix bug stopping scan data when new connection is made while scan head is still scanning
- fix bug affecting timing of laser during scanning that in rare cases decreased its intesity

## v16.1.6
### JS-50 Firmware
- improvement to internal calibration data

## v16.1.5
### C API
- fix memory leak in scanning code when `jsScanHeadSetMinimumEncoderTravel` is used
- fix bug in scanning code caused by unused variable

## v16.1.4
### C API
- remove internal dynamic memory allocation for profile buffering
- change internal profile buffering implementation; note with change, old data not read out is no longer overwritten by new data

## v16.1.3
### .NET API
- fix bug where point data could be overwritten if profiles were not read out fast enough
- reset `ProfileBufferOverflowed` flag on `StartScanning`

### JS-50 Firmware
- fix bug where IP address change would prevent connection

## v16.1.2
### C API
- fix socket close error on Windows

## v16.1.1
### C API
- fix linker error with `jsScanHeadGetAlignmentLaser` function

## v16.1.0
### .NET API
- add new scan head power cycle function `ScanHead.Reboot`
- add ability to set a `ScanWindow` per laser/camera
- add new polygonal window function `ScanWindow.CreateScanWindowPolygonal`
- add new scan data masking function `ScanHead.CreateExclusionMask` and `ScanHead.SetExclusionMask`
- add new beta feature brightness correction functions `ScanHead.CreateBrightnessCorrection` and `ScanHead.SetBrightnessCorrection` in the `JoeScan.Pinchot.Beta` namespace
- change datatype of X/Y data in `Point2D` from `double` to `float`
- fix handling of recoverable network errors
- various performance increases

### C API
- add new error codes `JS_ERROR_USE_CAMERA_FUNCTION` and `JS_ERROR_USE_LASER_FUNCTION`
- add new scan head power cycle function `jsPowerCycleScanHead`
- add new rectangular window functions `jsScanHeadSetWindowRectangularCamera` and `jsScanHeadSetWindowRectangularLaser`
- add new polygon window functions `jsScanHeadSetPolygonWindow`, `jsScanHeadSetPolygonWindowCamera`, & `jsScanHeadSetPolygonWindowLaser`
- add new scan data masking functions `jsScanHeadSetExclusionMaskCamera` & `jsScanHeadSetExclusionMaskLaser` and struct `jsExclusionMask`
- add new encoder driven scanning functions `jsScanHeadSetMinimumEncoderTravel` and `jsScanHeadSetIdleScanPeriod`
- add new beta feature brightness correction functions `jsScanHeadSetBrightnessCorrectionCamera_BETA` & `jsScanHeadSetBrightnessCorrectionLaser_BETA` and struct `jsBrightnessCorrection_BETA`

### JS-50 Firmware
- fix networking bug causing scan head to fail to negotiate link with some network switches
- fix autoexposure bug preventing it from working at fractional scan resolutions
- add support for exclusion masks
- add support for brightness correction

### FirmwareUpdate
- add ability to update scan heads in parallel
- fix bug preventing ScanSync updates

## v16.0.4
### JS-50 Firmware
- fix firmware bug that caused connection error

## v16.0.3
### .NET API
- report ethernet link speed `LinkSpeedMbps` in `DiscoveredDevice`

### C API
- report ethernet link speed `link_speed_mbps` in `jsDiscovered`
- deprecate `jsScanSystemPhaseInsertCameraConfiguration`, use `jsScanSystemPhaseInsertConfigurationCamera` instead
- deprecate `jsScanSystemPhaseInsertLaserConfiguration`, use `jsScanSystemPhaseInsertConfigurationLaser` instead

### JS-50 Firmware
- fix network negotiation failure with 100Base-T switches
- fix "Core" files not being served correctly in web server

## ScanSync
- firmware release v2.1
- support digital input signal up to 40 volts

## v16.0.2
### .NET API
- support for JS-50 MX

### C API
- support for JS-50 MX

### JS-50 Firmware
- support for JS-50 MX

## v16.0.1
### .NET API
- fix exception when scanning at high speeds for `netstandard2.0`

### SensorTester
- fix bug where non-`Enabled` heads would prevent scanning
- fix bug where scan heads loaded from configuration file wouldn't get an IP address

## v16.0.0
### .NET API
- fix `Disconnect` and `StopScanning` throwing exceptions when the connect is severed prior to call

### C API
- use TCP for streaming scan data to client over network
- rename `udp_packets_received` to `packets_received` in `jsProfile` & `jsRawProfile`
- rename `udp_packets_expected` to `packets_expected` in `jsProfile` & `jsRawProfile`

### FirmwareUpdate
- improve transfer speed of firmware file to scan head

## v15.0.4
### .NET API
- move scan head cable orientation from 'SetAlignment' to `Orientation` property
- support TCP profile data streaming

### C API
- move scan head cable alignment from `jsScanHeadSetAlignmentCamera` and `jsScanHeadSetAlignmentLaser` to `jsScanHeadSetCableOrientation`
- support TCP profile data streaming

### JS-50 Firmware
- fix sequence numbers returned with profile data; increments with completion of phase table
- support TCP profile data streaming

## v15.0.3
### .NET API
- fix bug where `GetValidXYPoints` would cause a hard loop on an `IProfile` returned from `GetDiagnosticProfile`

### JS-50 Firmware
- fix bug where diagnostic profiles had incorrect data orientation for X6B devices

## v15.0.2
### .NET API
- fix bug where TCP socket would fail to bind on connection

## v15.0.1
### JS-50 Firmware
- fix internal bug affecting X6B20 and X6B30 scan heads allowing only data from 2 lasers to be returned
- fix internal bug causing encoder value to always be returned as zero

## v15.0.0
### .NET API
- add new scan head network discovery routine along with new functions `DiscoverDevices` and `DiscoverDevicesAsync`
- add `DiscoveredDevice` class to report scan heads discovered on the network
- remove firmware version property from `ScanHeadStatus` class
- remove requirement of specifying `ProductType` when calling `CreateScanHead`
- add `Flags` to profile data to associate events with profile data
- add `UdpPacketsExpected` and `UdpPacketsReceived` to profile data to help diagnose network issues
- ScanSync inputs reported in `Flags` when ScanSync running v2.0.0 & above firmware
- add `Version` property to `ScanHead` class
- add new function `GetDiagnosticProfile` to get profiles on demand while not scanning
- rename `GetImage` to `GetDiagnosticImage`
- add ability to call `SetAlignment` and `SetWindow` while connected
- remove `Zeroconf` as a dependency

### AutoAlignTool
- add new bundled tool to assist with getting alignment measurements

### C API
- add new example application `00-discover`
- add new scan head network discovery routine along with new functions `jsScanSystemDiscover` and `jsScanSystemGetDiscovered`
- add `jsDiscovered` struct to report scan heads discovered on the network
- remove firmware version fields from `jsScanHeadStatus` struct
- remove requirement of specifying `jsScanHeadType` when calling `jsScanSystemCreateScanHead`
- change underlying type of `jsScanSystem` and `jsScanHead` from `void *` to `int64_t`
- add new errors `JS_ERROR_NETWORK` and `JS_ERROR_NOT_DISCOVERED` to `jsError`
- add `jsProfileFlags` to profile data to associate events with profile data
- ScanSync inputs reported in `jsProfileFlags` when ScanSync running v2.0.0 & above firmware
- add new function `jsScanHeadGetFirmwareVersion`
- remove deprecated function `jsScanHeadConfigure`
- add new functions `jsScanHeadGetDiagnosticProfileCamera` and `jsScanHeadGetDiagnosticProfileLaser` to get profiles on demand while not scanning
- rename `jsScanHeadGetCameraImage` to `jsScanHeadGetDiagnosticImage`
- add new functions `jsScanHeadGetDiagnosticImageCamera` and `jsScanHeadGetDiagnosticImageLaser` to get images on demand while not scanning

## JS-50 Firmware
- profile data is returned to client in the same order for both Camera A and Camera B

## JSDeviceFinder

## ScanCheck

## ScanSync
- initial support AUX Y, INDEX Z, and SYNC inputs
- initial support for reporting encoder faults

## SensorTester
- fix bug where phase table times wouldn't update with change in max laser on time

## v14.1.6
### .NET API
- bug fix to phase table to use new configuration settings when changed

### JS-50 Firmware
- bug fix to scan window region of interest calculation that would prevent scan data from being generated

## v14.1.5
### JS-50 Firmware
- fix bug in pixel precision over temperature feature causing random pixels to be seen brighter or darker

## v14.1.4
### JS-50 Firmware
- fix bug in firmware updater

## v14.1.3
### JS-50 Firmware
- fix internal bug in firmware preventing client communicating to scan head due to scan head not fully initializing

## v14.1.2
### .NET API
- disable Nagle's algorithm to improve TCP throughput

### C API
- disable Nagle's algorithm to improve TCP throughput
- explicitly set API calling convention

### SensorTester
- fix issue where scan heads would fail to load from json file
- add default phase tables when loading scan heads from json file

## v14.1.1
### .NET API
- remove `Status` property
- add `RequestStatus` function to better denote that the status has to be queried from the scan head
- allow obtaining status while scanning by calling `RequestStatus`
- add SourceLink to enable stepping into and viewing Pinchot source code from Visual Studio

### C API
- fix `jsScanSystemIsConnected` function to accurately report back connected while scanning
- allow obtaining status while scanning by calling `jsScanHeadGetStatus`

## v14.1.0
### .NET API
- add `SetAlignment` function that takes a `Laser`

### C API
- add `jsScanHeadSetAlignmentLaser` and `jsScanHeadGetAlignmentLaser` functions

## v14.0.0
### .NET API
- add `net6.0` framework, remove `netcoreapp3.1` framework, upgrade `netstandard2.0` to `netstandard2.1`
- remove `System.Memory` dependency, add dependency to `ZeroConf`
- add support for `ProductType.JS50X6B20` and `ProductType.JS50X6B30` scan heads
- add support for metric units; set using `ScanSystemUnits` when initializing `ScanSystem`
- require `ProductType` when calling `CreateScanHead`
- change `Camera` and `Laser` enums to have value `0` be invalid
- remove `Camera0`, `Camera1`, and `Laser0` enums
- rename enums in `DataFormat` to add clarity
- rename enums in `Encoder` to add clarity
- change `StartScanning` to take scan period rather than scan rate
- replace `GetMaxScanRate` with `GetMinScanPeriod` to account for change in `StartScanning`
- phasing set by phase table functions, `AddPhase` and `AddPhaseElement`, rather than scan offset
- replace `Profile` with new `IProfile` interface to allow users to create mock data
- add `SequenceNumber` to `IProfile`
- connect and configuration messages to scan head use TCP rather than UDP
- add example for JS-50 X6B, update existing examples
- change various data types to better match their intent
- use `yield` for proper enumeration in `GetValidXYPoints`
- add `Cameras` and `Lasers` properties to `ScanHead` to help with enumeration
- add `MinimumEncoderTravel` and `IdleScanPeriodUs` properties to `ScanHeadConfiguration` to disable or reduce the queueing of profiles when the `Main` encoder isn't moving
- add `Capabilities` property to get the physical features and limits of a `ScanHead`

### C API
- add support for `JS_SCAN_HEAD_JS50X6B20` and `JS_SCAN_HEAD_JS50X6B30` scan heads
- add support for metric units; set using `jsUnits` when initializing `jsScanSystem`
- require `jsScanHeadType` when initializing `jsScanHead`
- change `jsCamera` and `jsLaser` enums to have value `0` be invalid
- add new `jsError` enums `JS_ERROR_ALREADY_EXISTS`, `JS_ERROR_NO_MORE_ROOM`, `JS_ERROR_TYPE_COMPATIBILITY`, and `JS_ERROR_TYPE_COMPATIBILITY`
- rename enums in `jsDataFormat` to add clarity
- rename enums in `jsEncoder` to add clarity
- change `jsScanSystemStartScanning` to take scan period rather than scan rate
- phasing set by phase table functions, `jsScanSystemPhase*`, rather than scan offset
- add `sequence_number` to `jsProfile` and `jsRawProfile`
- connect and configuration messages to scan head use TCP rather than UDP

### FirmwareUpdate
- add support for new `.update` file format
- fix cancel button not actually canceling the update

### SensorTester
- add device finder to see scan heads on the network
- add ability to see and modify the phase table
- add ability to select which laser fires and for how long in image mode
- add red tint to areas not in the window in image mode

## v13.1.3
### JS-50 Firmware
- fix ScanSync packet handling in firmware

### C API
- fix bug in UDP socket receive where for some systems no data was returned
- fix bug in API where invalid data points were returned when using `JS_DATA_FORMAT_XY_FULL`, `JS_DATA_FORMAT_XY_HALF`, or `JS_DATA_FORMAT_XY_QUARTER` data formats

## v13.1.2
### JS-50 Firmware
- fix scan system synchronization timing bug in firmware

## v13.1.1
### JS-50 Firmware
- fix bug in pixel precision over temperature feature

### SensorTester
- fixed where `IsEnabled` was not honored when loading configuration from JSON file

## v13.1.0
### .NET API
- added .NET 5 as a target
- added `ClearProfiles` function to remove all profiles from internal buffer

### C API
- added `jsScanHeadClearProfiles` function to remove all profiles from internal buffer
- fix `jsScanHeadIsConnected` to allow it to show which scan heads connected and which didn't on failed return code from `jsScanSystemConnect`
- fix `jsScanSystemFree` to allow function to complete in less time
- remove `JS_DATA_FORMAT_CAMERA_IMAGE_FULL` from `jsDataFormat`
- fix bug where calling `jsStartScanning` immediately after calling `jsStopScanning` caused the program to crash

### JS-50 Firmware
- added support for new camera pixel precision feature over temperature

### SensorTester
- fixed profile upstream/downstream being swapped in **Alignment** mode (saved alignment from **Alignment** now be applied correctly when switching back to **Point / Brightness** mode)
- fixed crash when **Start Scanning** was pressed too quickly after **Stop Scanning** was pressed

## v13.0.2
### .NET API
- removed `Microsoft.CodeAnalysis.FxCopAnalyzers` dependency

### JS-50 Firmware
- added initial support for additional camera pixel precision feature

### SensorTester
- fix intermittent crash when clicking and dragging too quickly after refreshing alignment

## v13.0.1
### .NET API
- renamed "JS-50 SC" to "JS-50 WSC"
- changed throwing of general `Excpetion` type to more specific types, e.g., `InvalidOperationException`, to better reflect what went wrong

### C API
- renamed "JS-50 SC" to "JS-50 WSC"
- internal code optimizations to reduce time to receive and process incoming scan data

### JS-50 Firmware
- fixed laser and camera autoexposure

### SensorTester
- fixed issue where setting an invalid window would display multiple error messages
- fixed crash when attempting to add an existing serial number into scan system

## v13.0.0
### .NET API
- increase max scan rate supported to 4kHz
- fixed issue where the cable upstream/downstream alignment parameters were opposite with regards to the physical orientation of the cable
- added initial support for JS-50 SC
- allow public creation of Point2D (Pinchot API PR #1)
- minor optimizations for API internals (Pinchot API PR #2,3,4)
- replaced unused `Hardware` field in status packets with `Product` field that corresponds to the scan head's product type (see: `ProductType`)
- added new enums `Camera.CameraA` and `Camera.CameraB`
- deprecated enums `Camera.Camera0` and `Camera.Camera1`

### C API
- increase max scan rate supported to 4kHz
- fixed issue where the cable upstream/downstream alignment parameters were opposite with regards to the physical orientation of the cable
- added initial support for JS-50 SC
- added new enums `JS_CAMERA_A` and `JS_CAMERA_B`
- deprecated enums `JS_CAMERA_0` and `JS_CAMERA_1`

### FirmwareUpdate
- added ability to detect ScanSyncs that are on the network
- added ability to update ScanSyncs
- added ability to sort device list by clicking on the column headers
- display product type in device list
- open update.gz file location when program is started from the offical release software bundle

### JS-50 Firmware
- corrected error where Camera B was reported as numeric identifier zero and Camera A was reported as numeric identifier one

### SensorTester
- added initial support for JS-50 SC
- added continual refresh option for alignment mode
- colors of profile lines now have similar shades between cameras in a scan head

## v12.0.1
### JS-50 Firmware
- Added improvements to allow better firmware support in the future

## v12.0.0
### .NET API
- restrict maximum scan rate to 3200 Hz
- changed bounds for `MaxCameraExposureTime`, `MinCameraExposureTime`, and `DefaultCameraExposureTime` to be range of 15 to 2000000 microseconds
- changed bounds for `MaxLaserOnTime`, `MinLaserOnTime`, and `DefaultLaserOnTime` to be range of 15 to 650000 microseconds
- changed UDP connection type to be connectionless
- added check for valid parameter values to `CreateScanWindowRectangular()`
- added `ScanSystem.GetMaxScanRate()` method to determine the system's max scan rate based on window sizes and laser on time
- hid `ScanHeadStatus.MaxScanRate` property in favor of `ScanSystem.GetMaxScanRate()`
- precomputed transformation matrix values for camera-to-mill transform in `ProfileAssembler` 
- retargeted from .NET Core 3.0 to .NET Core 3.1 (in addition to netstandard2.0)
- retargeted examples from .NET Core 3.0 to .NET Core 3.1

### C API
- restrict maximum scan rate to 3200 Hz
- change bounds for `camera_exposure_time_max_us`, `camera_exposure_time_mix_us`, and `camera_exposure_time_def_us` to be range of 15 to 2000000 microseconds
- change bounds for `laser_exposure_time_max_us`, `laser_exposure_time_mix_us`, and `laser_exposure_time_def_us` to be range of 15 to 650000 microseconds
- remove `max_scan_rate` from `jsScanHeadStatus`
- added new function `jsScanSystemGetMaxScanRate`
- maximum scan rate is now also determined by configured laser on time
- when calling `jsScanSystemConnect`, client will issue repeated connect messages if no response is received and timeout is not reached
- added checks to API calls with arguments of type `double` to ensure they are valid

### FirmwareUpdate
- added version information to title bar
- added commit hash field to device list
- fixed choppy behavior of popup status window when updating multiple heads

### JS-50 Firmware
- performance optimizations for increased reliability and throughput
- fix bug that prevented a ScanSync from being detected if removed and added back

### SensorTester
- improvements to numeric input controls
- added units to plot axes
- added exception handling for user attempting to disconnect if connection has been lost
- added version information to title bar

## v11.0.1
### JS-50 Firmware
- fix scan head lockups when scanning for extended periods of time

## v11.0.0
### .NET API
- renamed API from JScanApi to Pinchot
- created open source release of API
- license software under 3-Clause BSD License
- created `CameraImage` class for images and removed `Image` property from `Profile` class
- fixed hard loop bug when scan head failed to connect causing high CPU usage

### C API
- created open source release of API
- license software under 3-Clause BSD License
- removed `camera_exposure_time_us` for `jsProfile` and `jsRawProfile`
- added `mainboard_temp` to `jsScanHeadStatus`
- fix `jsScanHeadConfigure` and `jsScanHeadSetWindowRectangular` to return `JS_ERROR_INVALID_ARGUMENT` when value is out of range
- fix `jsScanHeadGetProfiles`, `jsScanHeadGetRawProfiles` and `jsScanHeadGetCameraImage` to return `JS_ERROR_NULL_ARGUMENT` on null valued argument
- fix `jsScanHeadGetStatus` to return `JS_ERROR_SCANNING` when called during scanning
- fix `jsScanHeadGetStatus` to return `JS_ERROR_NOT_CONNECTED` when called while not connected

### SensorTester
- replaced "calibrate" wording with "alignment"
- fixed default window postition such that application now opens to full screen
- hid not-yet-implemented `AverageIntensity` setting

## v10.1.0
### .NET API
- increased UDP receive socket buffer size

### C API
- added new function `jsGetAPISemanticVersion`
- added `firmware_version_major`, `firmware_version_minor`, and `firmware_version_patch` to `jsScanHeadStatus`

### FirmwareUpdate
- added new icon

### JS-50 Firmware
- fix bug that caused the reported number of profiles sent to be off by one

### SensorTester
- added ability to run SensorTester from command line with a scan system JSON file parameter
- added new icon
- increased window limits

## v10.0.2
### C API
- fix Window's network initialization
- fix value returned within `camera_exposure_time_us` in `jsCameraImage`

## v10.0.1
### .NET API
- clarified scan system exception messages when trying to do something while in an invalid state

### JS-50 Firmware
- better handle connection requests from multiple clients - should reliably communicate with only most recent client

## v10.0.0
### .NET API
- removed ApiDemo project and replaced with ApiExamples projects
- added `ScanHead.GetCameraImage` method to retrieve an image from a camera
- `ScanHead.ScanHeadConfiguration` will initialize to default `ScanHeadConfiguration` instead of `null`
- `ScanHead.Alignment` will initialize to default `AlignmentParameters` instead of `null`
- removed Standard 4.6.1 and Standard 4.7.2 build targets
- report correct max scan rate in status
- changed version strategy to be based on semantic versioning (9.1 -> 10.0.0)

### C API
- replaced API examples with new applications demonstrating API usage
- fixed `jsScanHeadWaitUntilProfilesAvailable` timeout
- changed version strategy to be based on semantic versioning (9.1 -> 10.0.0)

### FirmwareUpdate
- added ability to update multiple scan heads at once

### JSCalibrate
- integrated JSCalibrate into SensorTester

### JS-50 Firmware
- fixed flaw that caused profiles to arrive out of order
- changed version strategy to be based on semantic versioning (9.1 -> 10.0.0)

### SensorTester
- configuration parameters can now be applied to scan heads individually
- improved UI for setting scan windows and alignment parameters
- default configuration settings match API default configuration settings
- fixed bug with IPAddresses not showing
- integrated JSCalibrate into SensorTester
- fixed bug with Laser Return Intensity viewing
- image mode now includes a drawn red line indicating where the scan head thinks the laser is

## v9.0
### C API
- added argument to `jsScanHeadSetAlignment` to allow correcting profile data according to cable alignment

### JSCalibrate
- added the ability to perform alignment with circular features

## v8.3
### .NET API
- `ScanHead.SetAlignment` now takes a `ScanHeadOrientation` parameter in place of `cableIsUpstream` bool
- renamed `ScanHeadStatus.CameraTemperatures` to `ScanHeadStatus.Temperatures` keyed with new `TemperatureSensor` enum
- removed all `DataFormat`s except `XYFull`, `XYFullLMFull`, `XYHalf`, `XYHalfLMHalf`, `XYQuarter`, and `XYQuarterLMQuarter`
- replaced `ScanHead.Configuration` property with `ScanHead.GetConfigurationClone` method
- added default values to `ScanHeadConfiguration`
- extensive API documentation updates

### C API
- removed all `jsDataFormat` except `XYFull`, `XYFullLMFull`, `XYHalf`, `XYHalfLMHalf`, `XYQuarter`, and `XYQuarterLMQuarter`
- fixed internal alignment calculation used for `jsScanHeadSetAlignment`
- added new function `jsScanHeadWaitUntilProfilesAvailable`
- added new function `jsScanHeadGetProfiles` and data type `jsProfile`
- added new defines `JS_SCAN_HEAD_PROFILES_MAX` and `JS_PROFILE_DATA_LEN`

### JS-50 Firmware
- fixed bug that allowed exposure times near scan rate causing profile data to become corrupted
- fixed bug that kept the ScanSync LED on the scan head in an always off state

### SensorTester
- fixed bug that crashed utility when adding, removing, then re-adding scan heads

## v8.2
### JS-50 Firmware
- added check at initialization of `scan_server` to ensure IP address is valid

## v8.1
### .NET API
- added ability to change scan head orientation via `ScanHead.SetAlignment()` `cableIsUpstream` parameter
- added equality methods and operators to `Point2d` struct
- added `ScanWindow.CreateScanWindowUnconstrained()` method to allow creation of unconstrained scan windows

### JSCalibrate
- added ability to change scan head orientation

### JS-50 Firmware
- fixed bug where encoder data from ScanSync was not being sent with scan data

### SensorTester
- added ability to change scan head orientation

## v8.0
### .NET API
- re-organized ApiDemo and added comments
- updated ApiDemo to reference JScanApi as a NuGet package
- added `set` accessor to `Point2D` class properties
- changed `Profile.RawPoints` property type from `IList` to `Span`
- added `Profile.ValidPointCount` property to retrieve the number of valid points in `Profile.RawPoints`
- added `Profile.GetValidXYPoints(Span<Point2D> validPoints)` method
- refactored connection scheme to use UDP broadcasting of scan head addresses instead of DNS lookup
- added `VersionInformation` class for retrieval of API version information
- fixed bug with `ScanWindow` cloning

### C API
- refactored connection scheme to use UDP broadcasting of scan head addresses instead of DNS lookup

### JS-50 Firmware
- refactored connection scheme to respond to UDP broadcast connect from .NET and C APIs

### SensorTester
- fixed bug with adding scan heads after scan heads have been loaded from a file

## v7.3
### .NET API
- added .NET Core 3.0 support to API
- migrated ApiDemo project to .NET Core 3.0

### FirmwareUpdate
- migrated to .NET Framework 4.7.2 and latest project file format

### JSCalibrate
- migrated to .NET Framework 4.7.2 and latest project file format

## v7.2
### .NET API
- fixed bug with `ScanHead.SetAlignment()` which didn't set camera1 parameters
- unconsumed profiles will be cleared when `ScanSystem.StartScanning()` is called
- added profile buffer size limit of 1000 profiles per scan head
- added requested scan rate versus max scan rate check to `ScanHead.StartScanning()`
- performance enhancements to profile assembly
- added PDF documentation for API
- added max scan rate check and minor improvements to ApiDemo

### C API
- check maximum scan rate for each scan head with given configuration during call to `jsScanSystemConnect`
- return error on call to `jsScanSystemStartScanning` if requested rate exceeds the maximum allowed for any scan head
- added `jsError` enumerated type and function `jsGetError` to convert error to string
- added additional checks to gracefully handle `NULL` valued arguments to API functions
- corrected `max_scan_rate` reported by `jsScanHeadStatus` to report maximum overall scan rate rather than the scan rate per camera
- fixed internal error that could cause received data from one scan head to be reported as coming from a different scan head when accessed through the API

### JSCalibrate
- correted bug in which alignment error was only calculated from one camera's data
- calibration is done on both cameras simultaneously with the same parameters

### SensorTester
- alignment is now done on both cameras simultaneously with the same parameters
- added File menu options to save and load Active Heads
- added an enabled checkbox to Active Heads to allow enabling/disabling of individual heads
- added exception handling on failure to connect to head(s)
- fixed bug with performance monitor not working on certain processors
- added exception handling on failure to start scanning

## v7.1
### JSCalibrate
- corrected bug with camera selection

### SensorTester
- corrected default exposure values for microsecond time scale

## v7.0
### .NET API
- refactored `ScanHeadStatus` to allow for variable length data; e.g. encoders
- enabled reading camera temperature in status packets

### C API
- enabled reading camera temperature in status packets

## v6.0
### .NET API
- initial release of beta API

### C API
- unified time units to microseconds for API calls
- report UDP packets expected and received for `jsRawProfile`
- report back length of valid brightness and geometry for `jsRawProfile`
- added support for image capture using `jsScanHeadGetCameraImage`

## v5.0
### .NET API
- added `SUBPIXEL_FULL_LM_FULL` resolution preset

### C API
- initial release of beta API

### SensorTester
- added Encoder count display
- changed profile point display colors for better contrast

